"""Contains all the data models used in inputs/outputs"""

from .attachment import Attachment
from .attachment_create import AttachmentCreate
from .attachment_update import AttachmentUpdate
from .bar_chart_adtl import BarChartAdtl
from .brainstorm import Brainstorm
from .brainstorm_create import BrainstormCreate
from .brainstorm_state import BrainstormState
from .brainstorm_update import BrainstormUpdate
from .burn_up_chart_adtl import BurnUpChartAdtl
from .chart import Chart
from .chart_type import ChartType
from .comment import Comment
from .comment_create import CommentCreate
from .comment_reaction import CommentReaction
from .comment_reaction_create import CommentReactionCreate
from .comment_reaction_update import CommentReactionUpdate
from .comment_update import CommentUpdate
from .dartboard import Dartboard
from .dartboard_create import DartboardCreate
from .dartboard_kind import DartboardKind
from .dartboard_update import DartboardUpdate
from .dartboards_list_kind import DartboardsListKind
from .dashboard import Dashboard
from .dashboard_create import DashboardCreate
from .dashboard_update import DashboardUpdate
from .discord_integration import DiscordIntegration
from .doc import Doc
from .doc_create import DocCreate
from .doc_source_type import DocSourceType
from .doc_update import DocUpdate
from .entity_name import EntityName
from .event import Event
from .event_actor import EventActor
from .event_create import EventCreate
from .event_kind import EventKind
from .event_subscription import EventSubscription
from .event_subscription_update import EventSubscriptionUpdate
from .filter_applicability import FilterApplicability
from .filter_assignee import FilterAssignee
from .filter_connector import FilterConnector
from .filter_group import FilterGroup
from .filter_search import FilterSearch
from .filter_set import FilterSet
from .folder import Folder
from .folder_create import FolderCreate
from .folder_kind import FolderKind
from .folder_update import FolderUpdate
from .folders_list_kind import FoldersListKind
from .form import Form
from .form_create import FormCreate
from .form_field import FormField
from .form_field_create import FormFieldCreate
from .form_field_update import FormFieldUpdate
from .form_update import FormUpdate
from .github_integration import GithubIntegration
from .github_integration_tenant_extension_status import GithubIntegrationTenantExtensionStatus
from .google_data import GoogleData
from .icon_kind import IconKind
from .layout import Layout
from .layout_config import LayoutConfig
from .layout_create import LayoutCreate
from .layout_kind import LayoutKind
from .layout_kind_config_map import LayoutKindConfigMap
from .layout_update import LayoutUpdate
from .line_chart_adtl import LineChartAdtl
from .models_response import ModelsResponse
from .notification import Notification
from .notification_update import NotificationUpdate
from .notion_integration import NotionIntegration
from .notion_integration_tenant_extension_status import NotionIntegrationTenantExtensionStatus
from .number_chart_adtl import NumberChartAdtl
from .number_chart_aggregation import NumberChartAggregation
from .operation import Operation
from .operation_kind import OperationKind
from .operation_model_kind import OperationModelKind
from .option import Option
from .option_create import OptionCreate
from .option_update import OptionUpdate
from .paginated_attachment_list import PaginatedAttachmentList
from .paginated_comment_list import PaginatedCommentList
from .paginated_comment_reaction_list import PaginatedCommentReactionList
from .paginated_dartboard_list import PaginatedDartboardList
from .paginated_dashboard_list import PaginatedDashboardList
from .paginated_doc_list import PaginatedDocList
from .paginated_folder_list import PaginatedFolderList
from .paginated_form_field_list import PaginatedFormFieldList
from .paginated_form_list import PaginatedFormList
from .paginated_layout_list import PaginatedLayoutList
from .paginated_option_list import PaginatedOptionList
from .paginated_property_list import PaginatedPropertyList
from .paginated_relationship_kind_list import PaginatedRelationshipKindList
from .paginated_relationship_list import PaginatedRelationshipList
from .paginated_space_list import PaginatedSpaceList
from .paginated_status_list import PaginatedStatusList
from .paginated_task_doc_relationship_list import PaginatedTaskDocRelationshipList
from .paginated_task_kind_list import PaginatedTaskKindList
from .paginated_task_link_list import PaginatedTaskLinkList
from .paginated_task_list import PaginatedTaskList
from .paginated_tenant_list import PaginatedTenantList
from .paginated_user_dartboard_layout_list import PaginatedUserDartboardLayoutList
from .paginated_user_list import PaginatedUserList
from .paginated_view_list import PaginatedViewList
from .paginated_webhook_list import PaginatedWebhookList
from .pie_chart_adtl import PieChartAdtl
from .pie_chart_display_metric import PieChartDisplayMetric
from .priority import Priority
from .properties_list_kind import PropertiesListKind
from .property_ import Property
from .property_create import PropertyCreate
from .property_kind import PropertyKind
from .property_update import PropertyUpdate
from .relationship import Relationship
from .relationship_create import RelationshipCreate
from .relationship_kind import RelationshipKind
from .relationship_kind_create import RelationshipKindCreate
from .relationship_kind_kind import RelationshipKindKind
from .relationship_kind_update import RelationshipKindUpdate
from .report_kind import ReportKind
from .request_body import RequestBody
from .response_body import ResponseBody
from .saml_config import SamlConfig
from .saml_config_tenant_extension_status import SamlConfigTenantExtensionStatus
from .slack_integration import SlackIntegration
from .slack_integration_tenant_extension_status import SlackIntegrationTenantExtensionStatus
from .sort import Sort
from .space import Space
from .space_create import SpaceCreate
from .space_kind import SpaceKind
from .space_update import SpaceUpdate
from .sprint_mode import SprintMode
from .status import Status
from .status_create import StatusCreate
from .status_kind import StatusKind
from .status_update import StatusUpdate
from .statuses_list_kind import StatusesListKind
from .subscription import Subscription
from .subtask_display_mode import SubtaskDisplayMode
from .summary_statistic_kind import SummaryStatisticKind
from .table_chart_adtl import TableChartAdtl
from .task import Task
from .task_create import TaskCreate
from .task_detail_mode import TaskDetailMode
from .task_doc_relationship import TaskDocRelationship
from .task_doc_relationship_create import TaskDocRelationshipCreate
from .task_kind import TaskKind
from .task_kind_create import TaskKindCreate
from .task_kind_kind import TaskKindKind
from .task_kind_update import TaskKindUpdate
from .task_kinds_list_kind import TaskKindsListKind
from .task_link import TaskLink
from .task_link_create import TaskLinkCreate
from .task_link_kind import TaskLinkKind
from .task_link_update import TaskLinkUpdate
from .task_notion_document import TaskNotionDocument
from .task_notion_document_block_children_map_type_0 import TaskNotionDocumentBlockChildrenMapType0
from .task_notion_document_block_map_type_0 import TaskNotionDocumentBlockMapType0
from .task_notion_document_page_map_type_0 import TaskNotionDocumentPageMapType0
from .task_properties import TaskProperties
from .task_source_type import TaskSourceType
from .task_update import TaskUpdate
from .tenant import Tenant
from .tenant_update import TenantUpdate
from .theme import Theme
from .transaction import Transaction
from .transaction_kind import TransactionKind
from .transaction_response import TransactionResponse
from .user import User
from .user_dartboard_layout import UserDartboardLayout
from .user_dartboard_layout_create import UserDartboardLayoutCreate
from .user_data_entity_retrieve_entity_kind import UserDataEntityRetrieveEntityKind
from .user_role import UserRole
from .user_status import UserStatus
from .user_update import UserUpdate
from .validation_error_response import ValidationErrorResponse
from .validation_error_response_items import ValidationErrorResponseItems
from .view import View
from .view_create import ViewCreate
from .view_kind import ViewKind
from .view_update import ViewUpdate
from .webhook import Webhook
from .webhook_create import WebhookCreate
from .webhook_update import WebhookUpdate
from .zapier_integration import ZapierIntegration

__all__ = (
    "Attachment",
    "AttachmentCreate",
    "AttachmentUpdate",
    "BarChartAdtl",
    "Brainstorm",
    "BrainstormCreate",
    "BrainstormState",
    "BrainstormUpdate",
    "BurnUpChartAdtl",
    "Chart",
    "ChartType",
    "Comment",
    "CommentCreate",
    "CommentReaction",
    "CommentReactionCreate",
    "CommentReactionUpdate",
    "CommentUpdate",
    "Dartboard",
    "DartboardCreate",
    "DartboardKind",
    "DartboardsListKind",
    "DartboardUpdate",
    "Dashboard",
    "DashboardCreate",
    "DashboardUpdate",
    "DiscordIntegration",
    "Doc",
    "DocCreate",
    "DocSourceType",
    "DocUpdate",
    "EntityName",
    "Event",
    "EventActor",
    "EventCreate",
    "EventKind",
    "EventSubscription",
    "EventSubscriptionUpdate",
    "FilterApplicability",
    "FilterAssignee",
    "FilterConnector",
    "FilterGroup",
    "FilterSearch",
    "FilterSet",
    "Folder",
    "FolderCreate",
    "FolderKind",
    "FoldersListKind",
    "FolderUpdate",
    "Form",
    "FormCreate",
    "FormField",
    "FormFieldCreate",
    "FormFieldUpdate",
    "FormUpdate",
    "GithubIntegration",
    "GithubIntegrationTenantExtensionStatus",
    "GoogleData",
    "IconKind",
    "Layout",
    "LayoutConfig",
    "LayoutCreate",
    "LayoutKind",
    "LayoutKindConfigMap",
    "LayoutUpdate",
    "LineChartAdtl",
    "ModelsResponse",
    "Notification",
    "NotificationUpdate",
    "NotionIntegration",
    "NotionIntegrationTenantExtensionStatus",
    "NumberChartAdtl",
    "NumberChartAggregation",
    "Operation",
    "OperationKind",
    "OperationModelKind",
    "Option",
    "OptionCreate",
    "OptionUpdate",
    "PaginatedAttachmentList",
    "PaginatedCommentList",
    "PaginatedCommentReactionList",
    "PaginatedDartboardList",
    "PaginatedDashboardList",
    "PaginatedDocList",
    "PaginatedFolderList",
    "PaginatedFormFieldList",
    "PaginatedFormList",
    "PaginatedLayoutList",
    "PaginatedOptionList",
    "PaginatedPropertyList",
    "PaginatedRelationshipKindList",
    "PaginatedRelationshipList",
    "PaginatedSpaceList",
    "PaginatedStatusList",
    "PaginatedTaskDocRelationshipList",
    "PaginatedTaskKindList",
    "PaginatedTaskLinkList",
    "PaginatedTaskList",
    "PaginatedTenantList",
    "PaginatedUserDartboardLayoutList",
    "PaginatedUserList",
    "PaginatedViewList",
    "PaginatedWebhookList",
    "PieChartAdtl",
    "PieChartDisplayMetric",
    "Priority",
    "PropertiesListKind",
    "Property",
    "PropertyCreate",
    "PropertyKind",
    "PropertyUpdate",
    "Relationship",
    "RelationshipCreate",
    "RelationshipKind",
    "RelationshipKindCreate",
    "RelationshipKindKind",
    "RelationshipKindUpdate",
    "ReportKind",
    "RequestBody",
    "ResponseBody",
    "SamlConfig",
    "SamlConfigTenantExtensionStatus",
    "SlackIntegration",
    "SlackIntegrationTenantExtensionStatus",
    "Sort",
    "Space",
    "SpaceCreate",
    "SpaceKind",
    "SpaceUpdate",
    "SprintMode",
    "Status",
    "StatusCreate",
    "StatusesListKind",
    "StatusKind",
    "StatusUpdate",
    "Subscription",
    "SubtaskDisplayMode",
    "SummaryStatisticKind",
    "TableChartAdtl",
    "Task",
    "TaskCreate",
    "TaskDetailMode",
    "TaskDocRelationship",
    "TaskDocRelationshipCreate",
    "TaskKind",
    "TaskKindCreate",
    "TaskKindKind",
    "TaskKindsListKind",
    "TaskKindUpdate",
    "TaskLink",
    "TaskLinkCreate",
    "TaskLinkKind",
    "TaskLinkUpdate",
    "TaskNotionDocument",
    "TaskNotionDocumentBlockChildrenMapType0",
    "TaskNotionDocumentBlockMapType0",
    "TaskNotionDocumentPageMapType0",
    "TaskProperties",
    "TaskSourceType",
    "TaskUpdate",
    "Tenant",
    "TenantUpdate",
    "Theme",
    "Transaction",
    "TransactionKind",
    "TransactionResponse",
    "User",
    "UserDartboardLayout",
    "UserDartboardLayoutCreate",
    "UserDataEntityRetrieveEntityKind",
    "UserRole",
    "UserStatus",
    "UserUpdate",
    "ValidationErrorResponse",
    "ValidationErrorResponseItems",
    "View",
    "ViewCreate",
    "ViewKind",
    "ViewUpdate",
    "Webhook",
    "WebhookCreate",
    "WebhookUpdate",
    "ZapierIntegration",
)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.icon_kind import IconKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="DashboardUpdate")


@_attrs_define
class DashboardUpdate:
    """
    Attributes:
        duid (str):
        accessible_by_team (Union[Unset, bool]):
        accessible_by_user_duids (Union[Unset, List[str]]):
        order (Union[Unset, str]):
        title (Union[Unset, str]):
        description (Union[Unset, str]):
        icon_kind (Union[Unset, IconKind]): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (Union[Unset, str]):
        color_hex (Union[Unset, str]):
        layout_duid (Union[Unset, str]):
        favorited_by_user_duids (Union[Unset, List[str]]):
        charts (Union[Unset, Any]):
    """

    duid: str
    accessible_by_team: Union[Unset, bool] = UNSET
    accessible_by_user_duids: Union[Unset, List[str]] = UNSET
    order: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    icon_kind: Union[Unset, IconKind] = UNSET
    icon_name_or_emoji: Union[Unset, str] = UNSET
    color_hex: Union[Unset, str] = UNSET
    layout_duid: Union[Unset, str] = UNSET
    favorited_by_user_duids: Union[Unset, List[str]] = UNSET
    charts: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        accessible_by_team = self.accessible_by_team

        accessible_by_user_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.accessible_by_user_duids, Unset):
            accessible_by_user_duids = self.accessible_by_user_duids

        order = self.order

        title = self.title

        description = self.description

        icon_kind: Union[Unset, str] = UNSET
        if not isinstance(self.icon_kind, Unset):
            icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji

        color_hex = self.color_hex

        layout_duid = self.layout_duid

        favorited_by_user_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.favorited_by_user_duids, Unset):
            favorited_by_user_duids = self.favorited_by_user_duids

        charts = self.charts

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if accessible_by_team is not UNSET:
            field_dict["accessibleByTeam"] = accessible_by_team
        if accessible_by_user_duids is not UNSET:
            field_dict["accessibleByUserDuids"] = accessible_by_user_duids
        if order is not UNSET:
            field_dict["order"] = order
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if icon_kind is not UNSET:
            field_dict["iconKind"] = icon_kind
        if icon_name_or_emoji is not UNSET:
            field_dict["iconNameOrEmoji"] = icon_name_or_emoji
        if color_hex is not UNSET:
            field_dict["colorHex"] = color_hex
        if layout_duid is not UNSET:
            field_dict["layoutDuid"] = layout_duid
        if favorited_by_user_duids is not UNSET:
            field_dict["favoritedByUserDuids"] = favorited_by_user_duids
        if charts is not UNSET:
            field_dict["charts"] = charts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        accessible_by_team = d.pop("accessibleByTeam", UNSET)

        accessible_by_user_duids = cast(List[str], d.pop("accessibleByUserDuids", UNSET))

        order = d.pop("order", UNSET)

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        _icon_kind = d.pop("iconKind", UNSET)
        icon_kind: Union[Unset, IconKind]
        if isinstance(_icon_kind, Unset):
            icon_kind = UNSET
        else:
            icon_kind = IconKind(_icon_kind)

        icon_name_or_emoji = d.pop("iconNameOrEmoji", UNSET)

        color_hex = d.pop("colorHex", UNSET)

        layout_duid = d.pop("layoutDuid", UNSET)

        favorited_by_user_duids = cast(List[str], d.pop("favoritedByUserDuids", UNSET))

        charts = d.pop("charts", UNSET)

        dashboard_update = cls(
            duid=duid,
            accessible_by_team=accessible_by_team,
            accessible_by_user_duids=accessible_by_user_duids,
            order=order,
            title=title,
            description=description,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_hex=color_hex,
            layout_duid=layout_duid,
            favorited_by_user_duids=favorited_by_user_duids,
            charts=charts,
        )

        dashboard_update.additional_properties = d
        return dashboard_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ZapierIntegration")


@_attrs_define
class ZapierIntegration:
    """
    Attributes:
        enabled (bool):
        linked_user_duids (List[str]):
    """

    enabled: bool
    linked_user_duids: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled

        linked_user_duids = self.linked_user_duids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "enabled": enabled,
                "linkedUserDuids": linked_user_duids,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled")

        linked_user_duids = cast(List[str], d.pop("linkedUserDuids"))

        zapier_integration = cls(
            enabled=enabled,
            linked_user_duids=linked_user_duids,
        )

        zapier_integration.additional_properties = d
        return zapier_integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FormFieldCreate")


@_attrs_define
class FormFieldCreate:
    """
    Attributes:
        duid (str):
        form_duid (str):
        property_duid (str):
        order (str):
        locked (Union[Unset, bool]):
        required (Union[Unset, bool]):
        hidden (Union[Unset, bool]):
        label (Union[Unset, str]):
        default (Union[Unset, Any]):
    """

    duid: str
    form_duid: str
    property_duid: str
    order: str
    locked: Union[Unset, bool] = UNSET
    required: Union[Unset, bool] = UNSET
    hidden: Union[Unset, bool] = UNSET
    label: Union[Unset, str] = UNSET
    default: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        form_duid = self.form_duid

        property_duid = self.property_duid

        order = self.order

        locked = self.locked

        required = self.required

        hidden = self.hidden

        label = self.label

        default = self.default

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "formDuid": form_duid,
                "propertyDuid": property_duid,
                "order": order,
            }
        )
        if locked is not UNSET:
            field_dict["locked"] = locked
        if required is not UNSET:
            field_dict["required"] = required
        if hidden is not UNSET:
            field_dict["hidden"] = hidden
        if label is not UNSET:
            field_dict["label"] = label
        if default is not UNSET:
            field_dict["default"] = default

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        form_duid = d.pop("formDuid")

        property_duid = d.pop("propertyDuid")

        order = d.pop("order")

        locked = d.pop("locked", UNSET)

        required = d.pop("required", UNSET)

        hidden = d.pop("hidden", UNSET)

        label = d.pop("label", UNSET)

        default = d.pop("default", UNSET)

        form_field_create = cls(
            duid=duid,
            form_duid=form_duid,
            property_duid=property_duid,
            order=order,
            locked=locked,
            required=required,
            hidden=hidden,
            label=label,
            default=default,
        )

        form_field_create.additional_properties = d
        return form_field_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.task_notion_document_block_children_map_type_0 import TaskNotionDocumentBlockChildrenMapType0
    from ..models.task_notion_document_block_map_type_0 import TaskNotionDocumentBlockMapType0
    from ..models.task_notion_document_page_map_type_0 import TaskNotionDocumentPageMapType0


T = TypeVar("T", bound="TaskNotionDocument")


@_attrs_define
class TaskNotionDocument:
    """
    Attributes:
        page_id (str):
        refreshing (bool):
        exists_and_access_granted (Union[None, bool]):
        last_refresh_at (Union[None, datetime.datetime]):
        page_map (Union['TaskNotionDocumentPageMapType0', None]):
        block_map (Union['TaskNotionDocumentBlockMapType0', None]):
        block_children_map (Union['TaskNotionDocumentBlockChildrenMapType0', None]):
    """

    page_id: str
    refreshing: bool
    exists_and_access_granted: Union[None, bool]
    last_refresh_at: Union[None, datetime.datetime]
    page_map: Union["TaskNotionDocumentPageMapType0", None]
    block_map: Union["TaskNotionDocumentBlockMapType0", None]
    block_children_map: Union["TaskNotionDocumentBlockChildrenMapType0", None]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.task_notion_document_block_children_map_type_0 import TaskNotionDocumentBlockChildrenMapType0
        from ..models.task_notion_document_block_map_type_0 import TaskNotionDocumentBlockMapType0
        from ..models.task_notion_document_page_map_type_0 import TaskNotionDocumentPageMapType0

        page_id = self.page_id

        refreshing = self.refreshing

        exists_and_access_granted: Union[None, bool]
        exists_and_access_granted = self.exists_and_access_granted

        last_refresh_at: Union[None, str]
        if isinstance(self.last_refresh_at, datetime.datetime):
            last_refresh_at = self.last_refresh_at.isoformat()
        else:
            last_refresh_at = self.last_refresh_at

        page_map: Union[Dict[str, Any], None]
        if isinstance(self.page_map, TaskNotionDocumentPageMapType0):
            page_map = self.page_map.to_dict()
        else:
            page_map = self.page_map

        block_map: Union[Dict[str, Any], None]
        if isinstance(self.block_map, TaskNotionDocumentBlockMapType0):
            block_map = self.block_map.to_dict()
        else:
            block_map = self.block_map

        block_children_map: Union[Dict[str, Any], None]
        if isinstance(self.block_children_map, TaskNotionDocumentBlockChildrenMapType0):
            block_children_map = self.block_children_map.to_dict()
        else:
            block_children_map = self.block_children_map

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "pageId": page_id,
                "refreshing": refreshing,
                "existsAndAccessGranted": exists_and_access_granted,
                "lastRefreshAt": last_refresh_at,
                "pageMap": page_map,
                "blockMap": block_map,
                "blockChildrenMap": block_children_map,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.task_notion_document_block_children_map_type_0 import TaskNotionDocumentBlockChildrenMapType0
        from ..models.task_notion_document_block_map_type_0 import TaskNotionDocumentBlockMapType0
        from ..models.task_notion_document_page_map_type_0 import TaskNotionDocumentPageMapType0

        d = src_dict.copy()
        page_id = d.pop("pageId")

        refreshing = d.pop("refreshing")

        def _parse_exists_and_access_granted(data: object) -> Union[None, bool]:
            if data is None:
                return data
            return cast(Union[None, bool], data)

        exists_and_access_granted = _parse_exists_and_access_granted(d.pop("existsAndAccessGranted"))

        def _parse_last_refresh_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_refresh_at_type_0 = isoparse(data)

                return last_refresh_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        last_refresh_at = _parse_last_refresh_at(d.pop("lastRefreshAt"))

        def _parse_page_map(data: object) -> Union["TaskNotionDocumentPageMapType0", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                page_map_type_0 = TaskNotionDocumentPageMapType0.from_dict(data)

                return page_map_type_0
            except:  # noqa: E722
                pass
            return cast(Union["TaskNotionDocumentPageMapType0", None], data)

        page_map = _parse_page_map(d.pop("pageMap"))

        def _parse_block_map(data: object) -> Union["TaskNotionDocumentBlockMapType0", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                block_map_type_0 = TaskNotionDocumentBlockMapType0.from_dict(data)

                return block_map_type_0
            except:  # noqa: E722
                pass
            return cast(Union["TaskNotionDocumentBlockMapType0", None], data)

        block_map = _parse_block_map(d.pop("blockMap"))

        def _parse_block_children_map(data: object) -> Union["TaskNotionDocumentBlockChildrenMapType0", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                block_children_map_type_0 = TaskNotionDocumentBlockChildrenMapType0.from_dict(data)

                return block_children_map_type_0
            except:  # noqa: E722
                pass
            return cast(Union["TaskNotionDocumentBlockChildrenMapType0", None], data)

        block_children_map = _parse_block_children_map(d.pop("blockChildrenMap"))

        task_notion_document = cls(
            page_id=page_id,
            refreshing=refreshing,
            exists_and_access_granted=exists_and_access_granted,
            last_refresh_at=last_refresh_at,
            page_map=page_map,
            block_map=block_map,
            block_children_map=block_children_map,
        )

        task_notion_document.additional_properties = d
        return task_notion_document

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.status_kind import StatusKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="StatusCreate")


@_attrs_define
class StatusCreate:
    """
    Attributes:
        duid (str):
        property_duid (str):
        kind (StatusKind): * `Unstarted` - UNSTARTED
            * `Started` - STARTED
            * `Blocked` - BLOCKED
            * `Finished` - FINISHED
            * `Canceled` - CANCELED
        order (str):
        locked (Union[Unset, bool]):
        title (Union[Unset, str]):
        color_hex (Union[Unset, str]):
        description (Union[Unset, str]):
    """

    duid: str
    property_duid: str
    kind: StatusKind
    order: str
    locked: Union[Unset, bool] = UNSET
    title: Union[Unset, str] = UNSET
    color_hex: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        property_duid = self.property_duid

        kind = self.kind.value

        order = self.order

        locked = self.locked

        title = self.title

        color_hex = self.color_hex

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "propertyDuid": property_duid,
                "kind": kind,
                "order": order,
            }
        )
        if locked is not UNSET:
            field_dict["locked"] = locked
        if title is not UNSET:
            field_dict["title"] = title
        if color_hex is not UNSET:
            field_dict["colorHex"] = color_hex
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        property_duid = d.pop("propertyDuid")

        kind = StatusKind(d.pop("kind"))

        order = d.pop("order")

        locked = d.pop("locked", UNSET)

        title = d.pop("title", UNSET)

        color_hex = d.pop("colorHex", UNSET)

        description = d.pop("description", UNSET)

        status_create = cls(
            duid=duid,
            property_duid=property_duid,
            kind=kind,
            order=order,
            locked=locked,
            title=title,
            color_hex=color_hex,
            description=description,
        )

        status_create.additional_properties = d
        return status_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LineChartAdtl")


@_attrs_define
class LineChartAdtl:
    """
    Attributes:
        x_property_duid (str):
        group_by_property_duid (Union[None, str]):
    """

    x_property_duid: str
    group_by_property_duid: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        x_property_duid = self.x_property_duid

        group_by_property_duid: Union[None, str]
        group_by_property_duid = self.group_by_property_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "xPropertyDuid": x_property_duid,
                "groupByPropertyDuid": group_by_property_duid,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        x_property_duid = d.pop("xPropertyDuid")

        def _parse_group_by_property_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        group_by_property_duid = _parse_group_by_property_duid(d.pop("groupByPropertyDuid"))

        line_chart_adtl = cls(
            x_property_duid=x_property_duid,
            group_by_property_duid=group_by_property_duid,
        )

        line_chart_adtl.additional_properties = d
        return line_chart_adtl

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.doc_source_type import DocSourceType
from ..models.icon_kind import IconKind
from ..models.report_kind import ReportKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="DocCreate")


@_attrs_define
class DocCreate:
    """
    Attributes:
        duid (str):
        source_user (Union[None, Unset, str]):
        source_type (Union[Unset, DocSourceType]): * `Unknown` - UNKNOWN
            * `Onboarding` - ONBOARDING
            * `Recommendation` - RECOMMENDATION
            * `GeneratedReport` - GENERATED_REPORT
            * `ChatGPT` - CHAT_GPT
            * `Application` - APPLICATION Default: DocSourceType.UNKNOWN.
        drafter_duid (Union[None, Unset, str]):
        in_trash (Union[Unset, bool]):
        folder_duid (Union[Unset, str]):
        report_kind (Union[None, ReportKind, Unset]):
        order (Union[Unset, str]):
        title (Union[Unset, str]):
        text (Union[Unset, Any]):
        text_markdown (Union[Unset, str]):
        edited_by_ai (Union[Unset, bool]):
        recommendation_duid (Union[None, Unset, str]):
        editor_duids (Union[Unset, List[str]]):
        subscriber_duids (Union[Unset, List[str]]):
        icon_kind (Union[Unset, IconKind]): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (Union[Unset, str]):
        color_hex (Union[Unset, str]):
    """

    duid: str
    source_user: Union[None, Unset, str] = UNSET
    source_type: Union[Unset, DocSourceType] = DocSourceType.UNKNOWN
    drafter_duid: Union[None, Unset, str] = UNSET
    in_trash: Union[Unset, bool] = UNSET
    folder_duid: Union[Unset, str] = UNSET
    report_kind: Union[None, ReportKind, Unset] = UNSET
    order: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    text: Union[Unset, Any] = UNSET
    text_markdown: Union[Unset, str] = UNSET
    edited_by_ai: Union[Unset, bool] = UNSET
    recommendation_duid: Union[None, Unset, str] = UNSET
    editor_duids: Union[Unset, List[str]] = UNSET
    subscriber_duids: Union[Unset, List[str]] = UNSET
    icon_kind: Union[Unset, IconKind] = UNSET
    icon_name_or_emoji: Union[Unset, str] = UNSET
    color_hex: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        source_user: Union[None, Unset, str]
        if isinstance(self.source_user, Unset):
            source_user = UNSET
        else:
            source_user = self.source_user

        source_type: Union[Unset, str] = UNSET
        if not isinstance(self.source_type, Unset):
            source_type = self.source_type.value

        drafter_duid: Union[None, Unset, str]
        if isinstance(self.drafter_duid, Unset):
            drafter_duid = UNSET
        else:
            drafter_duid = self.drafter_duid

        in_trash = self.in_trash

        folder_duid = self.folder_duid

        report_kind: Union[None, Unset, str]
        if isinstance(self.report_kind, Unset):
            report_kind = UNSET
        elif isinstance(self.report_kind, ReportKind):
            report_kind = self.report_kind.value
        else:
            report_kind = self.report_kind

        order = self.order

        title = self.title

        text = self.text

        text_markdown = self.text_markdown

        edited_by_ai = self.edited_by_ai

        recommendation_duid: Union[None, Unset, str]
        if isinstance(self.recommendation_duid, Unset):
            recommendation_duid = UNSET
        else:
            recommendation_duid = self.recommendation_duid

        editor_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.editor_duids, Unset):
            editor_duids = self.editor_duids

        subscriber_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.subscriber_duids, Unset):
            subscriber_duids = self.subscriber_duids

        icon_kind: Union[Unset, str] = UNSET
        if not isinstance(self.icon_kind, Unset):
            icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji

        color_hex = self.color_hex

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if source_user is not UNSET:
            field_dict["sourceUser"] = source_user
        if source_type is not UNSET:
            field_dict["sourceType"] = source_type
        if drafter_duid is not UNSET:
            field_dict["drafterDuid"] = drafter_duid
        if in_trash is not UNSET:
            field_dict["inTrash"] = in_trash
        if folder_duid is not UNSET:
            field_dict["folderDuid"] = folder_duid
        if report_kind is not UNSET:
            field_dict["reportKind"] = report_kind
        if order is not UNSET:
            field_dict["order"] = order
        if title is not UNSET:
            field_dict["title"] = title
        if text is not UNSET:
            field_dict["text"] = text
        if text_markdown is not UNSET:
            field_dict["textMarkdown"] = text_markdown
        if edited_by_ai is not UNSET:
            field_dict["editedByAi"] = edited_by_ai
        if recommendation_duid is not UNSET:
            field_dict["recommendationDuid"] = recommendation_duid
        if editor_duids is not UNSET:
            field_dict["editorDuids"] = editor_duids
        if subscriber_duids is not UNSET:
            field_dict["subscriberDuids"] = subscriber_duids
        if icon_kind is not UNSET:
            field_dict["iconKind"] = icon_kind
        if icon_name_or_emoji is not UNSET:
            field_dict["iconNameOrEmoji"] = icon_name_or_emoji
        if color_hex is not UNSET:
            field_dict["colorHex"] = color_hex

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        def _parse_source_user(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        source_user = _parse_source_user(d.pop("sourceUser", UNSET))

        _source_type = d.pop("sourceType", UNSET)
        source_type: Union[Unset, DocSourceType]
        if isinstance(_source_type, Unset):
            source_type = UNSET
        else:
            source_type = DocSourceType(_source_type)

        def _parse_drafter_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        drafter_duid = _parse_drafter_duid(d.pop("drafterDuid", UNSET))

        in_trash = d.pop("inTrash", UNSET)

        folder_duid = d.pop("folderDuid", UNSET)

        def _parse_report_kind(data: object) -> Union[None, ReportKind, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                report_kind_type_0 = ReportKind(data)

                return report_kind_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, ReportKind, Unset], data)

        report_kind = _parse_report_kind(d.pop("reportKind", UNSET))

        order = d.pop("order", UNSET)

        title = d.pop("title", UNSET)

        text = d.pop("text", UNSET)

        text_markdown = d.pop("textMarkdown", UNSET)

        edited_by_ai = d.pop("editedByAi", UNSET)

        def _parse_recommendation_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        recommendation_duid = _parse_recommendation_duid(d.pop("recommendationDuid", UNSET))

        editor_duids = cast(List[str], d.pop("editorDuids", UNSET))

        subscriber_duids = cast(List[str], d.pop("subscriberDuids", UNSET))

        _icon_kind = d.pop("iconKind", UNSET)
        icon_kind: Union[Unset, IconKind]
        if isinstance(_icon_kind, Unset):
            icon_kind = UNSET
        else:
            icon_kind = IconKind(_icon_kind)

        icon_name_or_emoji = d.pop("iconNameOrEmoji", UNSET)

        color_hex = d.pop("colorHex", UNSET)

        doc_create = cls(
            duid=duid,
            source_user=source_user,
            source_type=source_type,
            drafter_duid=drafter_duid,
            in_trash=in_trash,
            folder_duid=folder_duid,
            report_kind=report_kind,
            order=order,
            title=title,
            text=text,
            text_markdown=text_markdown,
            edited_by_ai=edited_by_ai,
            recommendation_duid=recommendation_duid,
            editor_duids=editor_duids,
            subscriber_duids=subscriber_duids,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_hex=color_hex,
        )

        doc_create.additional_properties = d
        return doc_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TableChartAdtl")


@_attrs_define
class TableChartAdtl:
    """
    Attributes:
        column_property_duid (str):
        row_property_duid (Union[None, str]):
    """

    column_property_duid: str
    row_property_duid: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        column_property_duid = self.column_property_duid

        row_property_duid: Union[None, str]
        row_property_duid = self.row_property_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "columnPropertyDuid": column_property_duid,
                "rowPropertyDuid": row_property_duid,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column_property_duid = d.pop("columnPropertyDuid")

        def _parse_row_property_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        row_property_duid = _parse_row_property_duid(d.pop("rowPropertyDuid"))

        table_chart_adtl = cls(
            column_property_duid=column_property_duid,
            row_property_duid=row_property_duid,
        )

        table_chart_adtl.additional_properties = d
        return table_chart_adtl

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
